# minimal Windows 3.1 style MIDI output
# Jrn Loviscach, 2003-05-13
# link with winmm.lib
# in StdAfx.h uncomment the line "#define WIN32_LEAN_AND_MEAN"



#include "stdafx.h"
#include <windows.h>

long midi(int status, int data1 = 0, int data2 = 0)
{
	return (data2<<16) | (data1<<8) | status;
}

int main(int argc, char* argv[])
{
	MMRESULT result;
	
	UINT midiOutDevs = midiOutGetNumDevs();
	printf("Number of MIDI out devices: %d\n", midiOutDevs);

	if(midiOutDevs == 0)
	{
		exit(-1);
	}

	MIDIOUTCAPS midiOutCaps;

	// look what devices we've got
	UINT_PTR uDeviceID;
	for(uDeviceID = 0; uDeviceID < midiOutDevs; ++uDeviceID)
	{
		result = midiOutGetDevCaps(uDeviceID,      
			&midiOutCaps, sizeof(MIDIOUTCAPS));
		if(result != MMSYSERR_NOERROR)
			exit(-1);

		char type[20];
		switch(midiOutCaps.wTechnology)
		{
			case MOD_MIDIPORT: strcpy(type, "MIDIPORT"); break;
			case MOD_SYNTH: strcpy(type, "SYNTH"); break;
			case MOD_SQSYNTH: strcpy(type, "SQSYNTH"); break;
			case MOD_FMSYNTH: strcpy(type, "FMSYNTH"); break;
			case MOD_MAPPER: strcpy(type, "MAPPER"); break;
			// the following two may not be defined in older headers
			// case MOD_WAVETABLE: strcpy(type, "WAVETABLE"); break;
			// case MOD_SWSYNTH: strcpy(type, "SWSYNTH"); break;
			default: strcpy(type, "unknown");
		}
		
		printf("Device %d is a %s with type \"%s\".\n",
			uDeviceID, midiOutCaps.szPname, type);
	}

	// the device we chose; ugly: fixed at 1
	uDeviceID = 1;

	HMIDIOUT hmo;

	result = midiOutOpen(&hmo, uDeviceID,
		NULL, NULL, CALLBACK_NULL);
	if(result != MMSYSERR_NOERROR)
		exit(-1);

	result = midiOutShortMsg(hmo, midi(0xB1, 1, 127));
	if(result != MMSYSERR_NOERROR)
		exit(-1);

	result = midiOutShortMsg(hmo, midi(0xC1, 17));
	if(result != MMSYSERR_NOERROR)
		exit(-1);

	result = midiOutShortMsg(hmo, midi(0x91, 40, 127));
	if(result != MMSYSERR_NOERROR)
		exit(-1);

	Sleep(1000); // bad idea to control timing

	result = midiOutShortMsg(hmo, midi(0xC0, 11));
	if(result != MMSYSERR_NOERROR)
		exit(-1);

	int i;
	for(i=32; i<100; ++i)
	{
		result = midiOutShortMsg(hmo, midi(0x90, i, 64));
		if(result != MMSYSERR_NOERROR)
			exit(-1);

		Sleep(50); // bad idea to control timing

		result = midiOutShortMsg(hmo, midi(0x80, i));
		if(result != MMSYSERR_NOERROR)
			exit(-1);
	}

	result = midiOutShortMsg(hmo, midi(0x81, 40));
	if(result != MMSYSERR_NOERROR)
		exit(-1);

	result = midiOutClose(hmo);
	if(result != MMSYSERR_NOERROR)
		exit(-1);

	printf("Thanks for listening.\n");
	return 0;
}
